// This program uses Direct Draw to play AVI and Anutodesk animator
// FLI/FLC animation flic files in fullscreen mode. It supports either
// 640 x 480 8 bit (256 color palette) or 320 by 200 (256 colors)
// sized animation files. In the case of the AVI format only the Video
// stream is processed and played as quickly as possible audio streams
// are ignored.  It is a "C" language example and thus needs to refer
// explicitly to the "vtbl" when calling COM interface methods.
//

//
// The following symbol must be defined so that the (G)lobal (U)nique
// (ID)entifiers used the "QueryInterface" method is defined in only one
// of the project files which include the "windows" header.
//
#define INITGUID

#include <windows.h>
#include <commdlg.h>
#include <direct.h>
#include <ddraw.h>

#include "resource.h"

//
//  Functions in other files whichload the next image in the movie or flic.
//

extern BOOL BlitFirstFlxFrame(char *);
extern BOOL BlitFirstAVIFrame(char *);
extern BOOL BlitNextFlcFrame(void);
extern BOOL BlitNextFliFrame(void);
extern BOOL BlitNextAviFrame(void);

#define FLC_type 0          //  Identify the type of animation file
#define FLI_type 1          //  (found from its filename extension
#define AVI_type 2

//
// Variables used globally
//

unsigned char   *Screen;                    // A pointer to the FLAT
                                            // ADDRESS space that
                                            // represents the SCREEN, this
                                            // is the primary display surface.

long            AnimationType=FLC_type;     // Default to FLC type animations

BOOL            bInRam=FALSE;               // Read one frame from disk and
                                            // show it be fore reading the
                                            // next, or read all the frames
                                            // into memory in their compressed
                                            // form and decompress to SCREEN
                                            // this shoudl be a little faster
                                            // but Windows has excellent
                                            // file caching that makes this
                                            // virtually unnecesary.

long            pitch;                      // DirectDraw tells us how many
                                            // bytes there are beween each
                                            // row in the SCREEN

unsigned char   p_red[255],                 // The 8 bit color palette from
                p_green[255],               // the flic or movie file. In this
                p_blue[255];                // example of the use of DirectDraw
                                            // we will exclusively use the
                                            // 8 bit paletted display mode.

//
// Variables used locally - All the DirectDraw specific code is in this file
//

static HWND                hWnd;                   // The applications window

static long                lTimer          = 0;
static BOOL                bActive         = FALSE;
static BOOL                bIsInitialized  = FALSE;
static BOOL                bDoubleBuffered = FALSE;
static BOOL                bUseTimer       = FALSE;
static BOOL                bStereo         = FALSE;
static PALETTEENTRY        pe[256];

static LPDIRECTDRAW        lpDD            = NULL; // Pointers to the
static LPDIRECTDRAW2       lpDD2           = NULL; // DirectDraw object
static LPDIRECTDRAWSURFACE lpDDSPrimary    = NULL; // interfaces.
static LPDIRECTDRAWSURFACE lpDDSSecondary  = NULL;
static LPDIRECTDRAWPALETTE lpDDPal         = NULL;

//
// Prototypes for local functions
//

static BOOL RestoreAll(void);
static void UpdateFrame(void);
static void RemoveAll(void);
static BOOL StartupFailed(HWND);
static BOOL StereoFailed(HWND);
static BOOL Initialize(HINSTANCE, int);
static BOOL ReadFlicIntoSurfaces(BOOL, char *);
static int  SelectFileName(char *, char *, HWND, HINSTANCE );
static LRESULT CALLBACK WindowProc(HWND,UINT, WPARAM, LPARAM);

//////////////// The application's main function ////////////////////////////

int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,
                     LPSTR lpCmdLine,int nCmdShow)
{
    MSG  msg;
    char FlicName[256];
    //
    // Get the filename of the file for the flic or movie we wish to
    // play, use the filename extension to determine the file type.
    //
    if(!SelectFileName(FlicName,
                       "Flic files (*.FLC *.FLI *.AVI)|*.fl?;*.avi|",
                       NULL,hInstance))return 0;
    if     (strstr(strupr(FlicName),".FLI") != NULL)AnimationType=FLI_type;
    else if(strstr(strupr(FlicName),".AVI") != NULL)AnimationType=AVI_type;
    else                                            AnimationType=FLC_type;
    //
    // Initialize the application by creating the DirectDraw objects and
    // surfaces and then switching the display adapter into the correct mode.
    // Either 640 x 480  or  320 by 200 (both resloutions use 256 colors).
    //
    if(!Initialize(hInstance,nCmdShow))return FALSE;
    //
    // Read the first frame from the flic or movie into display memory
    //
    ReadFlicIntoSurfaces(TRUE,FlicName);
    //
    // Update the display surface (After this point the first frame should
    // be visible irrespective of whether double buffering is in use or
    // the image is being written directly into video memory.
    //
    UpdateFrame();
    //
    // If a timer is being used to display the frames at regular times
    // create it now.
    //
    if(bUseTimer){
      lTimer=SetTimer(hWnd,1,33,NULL);
    }
    //
    // Main message loop
    //
    while(1){
        if(PeekMessage(&msg,NULL,0,0,PM_NOREMOVE)){
            if(!GetMessage(&msg,NULL,0,0)) {
                return msg.wParam;
            }
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        //
        // If not using a timer - draw the next frame.
        //
        else if(bActive && !bUseTimer){
          if(bDoubleBuffered)
            lpDD->lpVtbl->WaitForVerticalBlank(lpDD,DDWAITVB_BLOCKBEGIN,NULL);
          ReadFlicIntoSurfaces(FALSE,NULL);
          UpdateFrame();
        }
        else{
          WaitMessage();
        }
    }
    return 0;
}

/////////////////////////// Initialize the application //////////////////////

static BOOL Initialize(HINSTANCE hInstance,int nCmdShow){
 WNDCLASS            wc;
 DDSURFACEDESC       ddsd;
 HRESULT             ddrval;
 DDSCAPS             ddscaps;
 //
 // Register a class for a Windows window
 //
 wc.style=          CS_HREDRAW|CS_VREDRAW;
 wc.lpfnWndProc=    WindowProc;
 wc.cbClsExtra=     0;
 wc.cbWndExtra=     0;
 wc.hInstance=      hInstance;
 wc.hIcon=          LoadIcon(hInstance,MAKEINTRESOURCE(IDI_ICON1));
 wc.hCursor=        LoadCursor(NULL,IDC_ARROW);
 wc.hbrBackground=  NULL;
 wc.lpszMenuName=   NULL;
 wc.lpszClassName=  "DDPLAY";
 RegisterClass(&wc);
 //
 // Make a window that fills the desktop - this window will not be
 // seen because we will use DirectDraw to draw the movie so that it
 // fills the screen. The primary function of this window is to
 // process messages.
 //
 hWnd=CreateWindowEx(
     0,
     "DDPLAY",
     "DDPLAY",
     WS_POPUP,
     0,
     0,
     GetSystemMetrics(SM_CXSCREEN),
     GetSystemMetrics(SM_CYSCREEN),
     NULL,
     NULL,
     hInstance,
     NULL );
 if(!hWnd)return FALSE;
 ShowWindow(hWnd,nCmdShow);
 UpdateWindow(hWnd);
 //
 // Create the DirectDraw object and get a pointer to its interface.
 //
 ddrval=DirectDrawCreate(NULL,&lpDD,NULL);
 if(ddrval!=DD_OK)return StartupFailed(hWnd);
 //
 // DirectX2 has a new interface "DIRECTDRAW2" which provides
 // additional functionality over "DIRECTDRAW" to get a pointer
 // to its interface we call the "QueryInterface" method of the
 // DIRECTDRAW interface. (Note the use of "lpVtbl" to get a pointer
 // to the "QueryInterface" method - this is required for "C".)
 //
 ddrval = lpDD->lpVtbl->QueryInterface(lpDD,
                                       &IID_IDirectDraw2,  // GUID
                                       (LPVOID *)&lpDD2);
 if(ddrval != DD_OK)return StartupFailed(hWnd);
 //
 // Tell DirectDraw we want fullscreen and need exclusive access
 // to the video memory.
 //
 ddrval=lpDD->lpVtbl->SetCooperativeLevel(lpDD,hWnd,
                                          DDSCL_EXCLUSIVE|
                                          DDSCL_FULLSCREEN);
 //
 // If the older FLI type flic is to be shown then put the
 // display into 320 x 200 resolution mode.
 //
 if(AnimationType == FLC_type || AnimationType == AVI_type){
   ddrval=lpDD2->lpVtbl->SetDisplayMode(lpDD2,640,480,8,0,0);
 }
 else if(AnimationType == FLI_type){
   ddrval=lpDD2->lpVtbl->SetDisplayMode(lpDD2,320,200,8,0,0);
 }
 if(ddrval!=DD_OK) return StartupFailed(hWnd);
 //
 // To draw on the screen we need a "Surface". There must be at least
 // one, the PRIMARY surface. It is an area of video memory and what is
 // so wonderful about DirectDraw this memory is provided as a "flat
 // address space" which can be addressed in the same way that one used
 // to use the "A000:0000" video memory area in DOS.
 //
 // Direct draw allows the creation of other "surfaces" in the display
 // adapter's video memory. These can be blitted onto the primary surface
 // using various ROP modes and in the context of this example such
 // a second surface will allow us to implement double buffering.
 //
 ddsd.dwSize=sizeof(ddsd);
 ddsd.dwFlags=DDSD_CAPS;
 ddsd.ddsCaps.dwCaps=DDSCAPS_PRIMARYSURFACE;
 ddrval=lpDD->lpVtbl->CreateSurface(lpDD,&ddsd,&lpDDSPrimary,NULL);
 if(ddrval != DD_OK)return StartupFailed(hWnd);
 //
 // Create a second surface if double buffering is in use.
 //
 if(bDoubleBuffered){
   ddsd.dwSize=sizeof(ddsd);
   ddsd.dwFlags=DDSD_CAPS|DDSD_HEIGHT|DDSD_WIDTH;
   ddsd.ddsCaps.dwCaps=DDSCAPS_OFFSCREENPLAIN;
   if(AnimationType == FLC_type || AnimationType == AVI_type){
    ddsd.dwWidth=640; ddsd.dwHeight=480;
   }
   else if(AnimationType == FLI_type){
     ddsd.dwWidth=320; ddsd.dwHeight=200;
   }
   lpDD->lpVtbl->CreateSurface(lpDD,&ddsd,&lpDDSSecondary,NULL);
   if(lpDDSSecondary==NULL)return StartupFailed(hWnd);
 }
 if(bStereo){
   ddsd.dwSize=sizeof(ddsd);
   ddsd.dwFlags=DDSD_CAPS|DDSD_HEIGHT|DDSD_WIDTH;
   ddsd.ddsCaps.dwCaps=DDSCAPS_PRIMARYSURFACELEFT;
   if(AnimationType == FLC_type || AnimationType == AVI_type){
    ddsd.dwWidth=640; ddsd.dwHeight=480;
   }
   else if(AnimationType == FLI_type){
     ddsd.dwWidth=320; ddsd.dwHeight=200;
   }
   ddrval=lpDD->lpVtbl->CreateSurface(lpDD,&ddsd,&lpDDSSecondary,NULL);
   if(ddrval != DD_OK)return StartupFailed(hWnd);
   if(lpDDSSecondary==NULL)return StereoFailed(hWnd);
   ddrval=lpDDSPrimary->lpVtbl->AddAttachedSurface(lpDDSPrimary,lpDDSSecondary);
   if(ddrval != DD_OK)return StartupFailed(hWnd);
 }

 bIsInitialized = TRUE;
 //
 // All the necessary DirectDraw objects have been created and initialized.
 //
 return TRUE;
}

///////////////////// The window function procedure /////////////////////////

static LRESULT CALLBACK WindowProc(HWND hWnd,UINT message,
                                   WPARAM wParam,LPARAM lParam){
    int speed;
    switch(message)
    {
        case WM_TIMER:
          if(bActive){
            //
            // The timer has gone off - read the next image (frame)
            // into the DirectDraw surface and display it.
            //
            if(bDoubleBuffered)lpDD->lpVtbl->WaitForVerticalBlank(lpDD,
                                           DDWAITVB_BLOCKBEGIN,NULL);
            ReadFlicIntoSurfaces(FALSE,NULL);
            UpdateFrame();
          }
          break;
        case WM_ACTIVATE:
            bActive = wParam;
            break;
        case WM_CREATE:
            break;
        case WM_SETCURSOR:
            SetCursor(NULL);
            if( bIsInitialized )
            {
                UpdateFrame();
                lpDDPal->lpVtbl->GetEntries(lpDDPal,0,0,256,pe);
            }
            break;

        case WM_KEYDOWN:
            switch(wParam) {
                //
                // Use the ESCAPE or F12 key to terminate the application.
                //
                case VK_ESCAPE:
                case VK_F12:
                  PostMessage(hWnd,WM_CLOSE,0,0);
                  break;
                case VK_F1: speed=1;    goto SPEED;
                case VK_F2: speed=5;    goto SPEED;
                case VK_F3: speed=10;   goto SPEED;
                case VK_F4: speed=20;   goto SPEED;
                case VK_F5: speed=33;   goto SPEED;
                case VK_F6: speed=66;   goto SPEED;
                case VK_F7: speed=132;  goto SPEED;
                case VK_F8: speed=266;  goto SPEED;
                  //
                  // If a timer is in use use these keys to
                  // change the display rate.
                  //
                  SPEED:
                  if(lTimer > 0){
                    KillTimer(hWnd,lTimer);
                    lTimer=SetTimer(hWnd,1,speed,NULL);
                  }
                  break;
            }
            break;
        case WM_DESTROY:
            if(lTimer > 0)KillTimer(hWnd,lTimer);
            RemoveAll();
            PostQuitMessage(0);
            break;
    }
    return DefWindowProc(hWnd,message,wParam,lParam);
}

//////////////////////// The remaining DirectDraw functions /////////////////

static BOOL ReadFlicIntoSurfaces(BOOL first, char *FlicName){
 HRESULT             ddrval;
 PALETTEENTRY        pe[256];
 DDSURFACEDESC       DDSDesc;
 int                 i;
 //
 // The next image is composed in this function. The main task is
 // to assign a standard "BYTE" type pointer to the block of memory
 // into which the image will be assembled by an approproate function.
 //
 // Direct3D insists that the block of memory we want to use is "Locked"
 // before we use it. Locking a block of memory informs us of the
 // "pitch" we must use in addressing the video memory on a row by
 // row basis. After the memory buffer has been used we must unlock it.
 //
 DDSDesc.dwSize=sizeof(DDSDesc);
 if(bDoubleBuffered)
   ddrval=lpDDSSecondary->lpVtbl->Lock(lpDDSSecondary,NULL,&DDSDesc,0,NULL);
 else
   ddrval=lpDDSPrimary->lpVtbl->Lock(lpDDSPrimary,NULL,&DDSDesc,0,NULL);
 if(ddrval!=DD_OK)return FALSE;
 //
 // Assign values to the global variables for pitch and memory buffer.
 // (These are used in the other files of the project. For an AVI movie
 // the image read from the video stream is copied to "Screen").
 //
 pitch=DDSDesc.lPitch;
 Screen=(unsigned char *)DDSDesc.lpSurface;
 //
 // If this is the first time this function is called the first frame
 // in the animation is displayed.
 //
 if(first){
   //
   // Copy the first frame into video memory.
   //
   if(AnimationType == AVI_type)BlitFirstAviFrame(FlicName);
   else                         BlitFirstFlxFrame(FlicName);
   //
   // Unlock the memory after we have written to it.
   //
   if(bDoubleBuffered)
     lpDDSSecondary->lpVtbl->Unlock(lpDDSSecondary,NULL);
   else
     lpDDSPrimary->lpVtbl->Unlock(lpDDSPrimary,NULL);
   //
   // Because the display is set to a paletted mode we must create
   // and use a color palette. Flic files contain such a palette and that
   // is read into global variables "p_red[]" etc. during execution of
   // BlitFirst???Frame(name);
   //
   for(i=0;i<256;i++){
     pe[i].peRed=p_red[i];
     pe[i].peGreen=p_green[i];
     pe[i].peBlue=p_blue[i];
   }
   //
   // Create the DirectDraw palette and put it to use.
   //
   ddrval=lpDD->lpVtbl->CreatePalette(lpDD,DDPCAPS_8BIT,pe,&lpDDPal,NULL);
   if(ddrval!=DD_OK)return FALSE;
   lpDDSPrimary->lpVtbl->SetPalette(lpDDSPrimary,lpDDPal);
 }
 else{
   //
   // Read the next image into video memory
   //
   if     (AnimationType == FLC_type)BlitNextFlcFrame();
   else if(AnimationType == AVI_type)BlitNextAviFrame();
   else                              BlitNextFliFrame();
   //
   // Unlock the memory buffer after we have written to it.
   //
   if(bDoubleBuffered)
     lpDDSSecondary->lpVtbl->Unlock(lpDDSSecondary,NULL);
   else
     lpDDSPrimary->lpVtbl->Unlock(lpDDSPrimary,NULL);
 }
 return TRUE;
}

static void UpdateFrame(void){
 HRESULT             ddrval;
 RECT                rcRect;
 RECT                destRect;
 POINT               pt;
 if(bStereo){
   lpDDSPrimary->lpVtbl->Flip(lpDDSPrimary,NULL,0);

 }
 if(!bDoubleBuffered)return;
 //
 // If double buffering is in use the secondary surface must be
 // blitted to the primary surface. The "Blt" method requires
 // a source and destination rectange and these are determined
 // before executing the blit.
 //
 rcRect.left=0;
 rcRect.top=0;
 if(AnimationType == FLC_type || AnimationType == AVI_type){
   rcRect.right=640; rcRect.bottom=480;
 }
 else if(AnimationType == FLI_type){
   rcRect.right=320; rcRect.bottom=200;
 }
 GetClientRect(hWnd,&destRect);
 pt.x=pt.y=0;
 ClientToScreen(hWnd,&pt);
 OffsetRect(&destRect,pt.x,pt.y);
 while(1){
   //
   // Continue trying to execute the blit if the device is busy
   // and an error has not occured.
   //
   ddrval=lpDDSPrimary->lpVtbl->Blt(lpDDSPrimary,
                                    &destRect,
                                    lpDDSSecondary,
                                    &rcRect,
                                    DDBLT_WAIT,NULL);
   if(ddrval==DD_OK)break;
   if(ddrval==DDERR_SURFACELOST){
     if(!RestoreAll())return;
     continue;
   }
   if(ddrval!=DDERR_WASSTILLDRAWING)return;
 }
}

static BOOL RestoreAll(void){
 //
 // It is standard practice in direct draw to restore the surface if
 // it should be lost (given to some other process). For our example
 // we don't need to redraw the image because this will be done
 // automatically when the next frame is shown.
 //
 BOOL bResult;
 bResult=lpDDSPrimary->lpVtbl->Restore(lpDDSPrimary) == DD_OK &&
         lpDDSSecondary->lpVtbl->Restore(lpDDSSecondary) == DD_OK;
 return(bResult);
}

static void RemoveAll(void){
 //
 // Release all the DirectDraw objects
 //
 if(lpDD!=NULL) {
   //
   // Put the display back to normal windowed mode.
   //
   lpDD->lpVtbl->SetCooperativeLevel(lpDD,hWnd, DDSCL_NORMAL);
   lpDD2->lpVtbl->RestoreDisplayMode(lpDD2);
   //
   // Release the surfaces, palette and DirectDraw object (the display).
   //
   if(lpDDSPrimary!=NULL) {
     lpDDSPrimary->lpVtbl->Release(lpDDSPrimary);
     lpDDSPrimary=NULL;
   }
   if(lpDDSSecondary!=NULL) {
     lpDDSSecondary->lpVtbl->Release(lpDDSSecondary);
     lpDDSSecondary=NULL;
   }
   if(lpDDPal!=NULL) {
     lpDDPal->lpVtbl->Release(lpDDPal);
     lpDDPal=NULL;
   }
   lpDD->lpVtbl->Release(lpDD);
   lpDD=NULL;
 }
 return;
}


static BOOL StartupFailed(HWND hWnd){
    RemoveAll();
    MessageBox(hWnd,"DirectDraw could not be intialized",NULL,MB_OK);
    DestroyWindow(hWnd);
    return FALSE;
}

static BOOL StereoFailed(HWND hWnd){
    RemoveAll();
    MessageBox(hWnd,"DirectDraw could not do Stereo",NULL,MB_OK);
    DestroyWindow(hWnd);
    return FALSE;
}


/////////////////////////////////////////////////////////////////////////////

//
// The last two functions provide the "User Interface", it is based on
// an expanded "Explorer Style File Open Dialog" that provides extra check
// boxes that enable the user to set the flags:
//                                             bDoubleBuffered,
//                                             bUserTimer,
//                                             bInRam

static unsigned CALLBACK NameHookProc(HWND hwnd,UINT msg,
                              WPARAM wparam,LPARAM lparam){
 //
 // Callback function used by the GetOpenFileName API
 //
 switch( msg ) {
   case WM_INITDIALOG:
     if(bInRam)SendDlgItemMessage(hwnd,IDC_CHECK1,BM_SETCHECK,1,0);
     if(bDoubleBuffered)SendDlgItemMessage(hwnd,IDC_CHECK2,BM_SETCHECK,1,0);
     if(bUseTimer)SendDlgItemMessage(hwnd,IDC_CHECK2,BM_SETCHECK,1,0);
     return FALSE;
   case WM_NOTIFY:{
       LPOFNOTIFY lp=(LPOFNOTIFY)lparam;
       if(lp->hdr.code == CDN_FILEOK){
         //
         // OK is pressed get the flags
         //
         if(SendDlgItemMessage(hwnd,IDC_CHECK1,BM_GETCHECK,0,0))
              bInRam=TRUE;
         else bInRam=FALSE;
         if(SendDlgItemMessage(hwnd,IDC_CHECK2,BM_GETCHECK,0,0))
              bDoubleBuffered=TRUE;
         else bDoubleBuffered=FALSE;
         if(SendDlgItemMessage(hwnd,IDC_CHECK3,BM_GETCHECK,0,0))
              bUseTimer=TRUE;
         else bUseTimer=FALSE;
       }
     }
     break;
   default: break;
 }
 return FALSE;
}

static BOOL SelectFileName(char *szfile, char *szfilter,
                           HWND parent, HINSTANCE hInst){
 //
 // Get a filename or the movie or flic file by setting up an OPENFILENAME
 // structure and calling the API "GetOpenFileName"
 //
 int i;
 OPENFILENAME ofn;
 char szFilter[80];
 strcpy(szFilter,szfilter);
 i=0; while(szFilter[i] != '\0'){
   if(szFilter[i] == '|')szFilter[i]='\0'; i++;
 }
 memset(&ofn,0,sizeof(OPENFILENAME));
 ofn.lStructSize=sizeof(OPENFILENAME);
 ofn.hwndOwner=parent;
 ofn.lpstrFilter=szFilter;
 ofn.nFilterIndex=0;
 szfile[0]='\0';
 ofn.lpstrFile=szfile;
 ofn.nMaxFile=255;
 ofn.lpstrFileTitle=NULL;
 ofn.nMaxFileTitle=0;
 ofn.lpstrInitialDir=NULL;
 ofn.lpstrTitle="Open FLIC to Play Full Screen";
 ofn.hInstance=hInst;
 ofn.lpfnHook=NameHookProc;
 ofn.lpTemplateName=MAKEINTRESOURCE(IDD_DIALOG1);
 ofn.Flags=OFN_HIDEREADONLY|OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST|
           OFN_EXPLORER|
           OFN_ENABLEHOOK|OFN_ENABLETEMPLATE|OFN_NONETWORKBUTTON;
 return GetOpenFileName(&ofn);
}

