/* Code to decode and fill a memory buffer with images obtained from */
/* a FLIC file.                                                      */

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "FLChead.h"
#include "FLIhead.h"

#define lMalloc(s)    LocalAlloc(LMEM_FIXED,s)
#define lFree(p)      LocalFree((HLOCAL)p)
#define lRealloc(p,s) LocalReAlloc((HLOCAL)p,s,LMEM_MOVEABLE)

#define FLC_type 0
#define FLI_type 1
#define AVI_type 2

#define PALVERSION  0x300
#define MAXPALETTE  256
#define MAXTEXT     256

//
// If storing the animation in ram we need a structure in which
// to record information about the frame and the frame data.
//

typedef struct tag_ram_frame{
 char *frame_header;
 char *frame;
}ram_frame;

static ram_frame *RamFrames     = NULL;
static FILE      *FpF           = NULL;
static Flc_head  flc_cel;
static Fli_head  fli_cel;

extern unsigned char p_red[],p_green[],p_blue[];
extern unsigned char *Screen;
extern long pitch;
extern BOOL bInRam;
extern long AnimationType;

BOOL BlitFirstFlxFrame(char *CurrentFile);
BOOL BlitFlcFrame(void);
BOOL BlitFliFrame(void);

static int nframes=0;
static long nx,ny,frame2start;
static long current_frame,max_frames=0;
static BOOL AnimationOpen=FALSE,AnimationPlaying=FALSE;

static BOOL LoadAnimationIntoRam(void);
static void FreeAnimation(void);
static BOOL FlcBrun(char * );
static BOOL FliBrun(char * );
static BOOL FlcColor(char *, short );
static BOOL FliColor(char * );
static BOOL LoadFirstFlcCel(FILE *, Flc_head *, int );
static BOOL flc_unlccomp(char *);
static BOOL fli_unlccomp(char *);
static BOOL OpenFLC(char *, int );
static BOOL OpenFLI(char *, int );

static void FreeAnimation(void){
 if(FpF != NULL)fclose(FpF);
 FpF            = NULL;
 AnimationOpen  = FALSE;
 if(RamFrames != NULL && nframes > 0){
   int i;
   for(i=0;i<nframes;i++){
     if(RamFrames[i].frame_header != NULL)lFree(RamFrames[i].frame_header);
     if(RamFrames[i].frame != NULL)lFree(RamFrames[i].frame);
   }
   lFree(RamFrames);
   RamFrames=NULL; nframes=0;
 }
}

static BOOL OpenFLC(char *name, int info_only){
 if((FpF=fopen(name,"rb")) == NULL)return FALSE;
 if(fread(&flc_cel,sizeof(Flc_head),1,FpF) != 1)return FALSE;
 if(flc_cel.type != FLCH_MAGIC)return FALSE;
 nx = flc_cel.width;
 ny = flc_cel.height;
 return LoadFirstFlcCel(FpF,&flc_cel,info_only);
}

static BOOL LoadFirstFlcCel(FILE *f,Flc_head *flc, int info_only){
 Flc_frame f_frame;
 Flc_chunk *f_chunk;
 long i,j,s;
 char c,*frame,*buff;
 unsigned char *p;
 fseek(f,flc->oframe1,SEEK_SET);
 if(fread(&f_frame,sizeof(Flc_frame),1,f) != 1 ||
           f_frame.type != FLCF_MAGIC)return FALSE;
 s=f_frame.size-16;
 if((frame = (char *)lMalloc(s)) == NULL)return FALSE;
 if(fread(frame,1,s,FpF) != s)return FALSE;
 buff=frame;
 for(i=0;i<f_frame.chunks;i++){
   f_chunk=(Flc_chunk *)buff;  buff += 6;
   if(f_chunk->type == FLC_COLOR256 || f_chunk->type == FLC_COLOR){
     if(!FlcColor(buff,f_chunk->type)){
       lFree(frame);
       return FALSE;
     }
     if(info_only)goto XXIT;
   }
   else if(!info_only && f_chunk->type == FLC_BRUN){
     if(!FlcBrun(buff)){
       lFree(frame);
       return FALSE;
     }
   }
   else if(!info_only && f_chunk->type == FLC_COPY){
     for (j=0;j<ny;j++){
       p=Screen+(j)*pitch;
       memcpy(p,buff+j*nx,nx);
     }
   }
   buff += (f_chunk->size - 6);
 }
 XXIT:
 lFree(frame);
 frame2start=ftell(f);
 return TRUE;
}

static BOOL OpenFLI(char *name, int info_only){
 if((FpF=fopen(name,"rb")) == NULL)return FALSE;
 if(fread(&fli_cel,sizeof(Fli_head),1,FpF) != 1)return FALSE;
 if(fli_cel.type != FLIH_MAGIC){
   return FALSE;
 }
 nx = fli_cel.width;
 ny = fli_cel.height;
 return LoadFirstFliCel(FpF,&fli_cel,info_only);
}

static BOOL LoadFirstFliCel(FILE *f,Fli_head *fli, int info_only){
 Fli_frame f_frame;
 Fli_chunk *f_chunk;
 long i,j,s;
 char c,*frame,*buff;
 unsigned char *p;
 if(fread(&f_frame,sizeof(Fli_frame),1,f) != 1){
   return FALSE;
 }
 if(f_frame.type != FLIF_MAGIC){
   return FALSE;
 }
 s=f_frame.size-16;
 if((frame = (char *)lMalloc(s)) == NULL)return FALSE;
 if(fread(frame,1,s,FpF) != s)return FALSE;
 buff=frame;
 for(i=0;i<f_frame.chunks;i++){
   f_chunk=(Fli_chunk *)buff;  buff += 6;
   if(f_chunk->type == FLI_COLOR){
     if(!FliColor(buff)){
       lFree(frame);
       return FALSE;
     }
     if(info_only)goto XXIT;
   }
   else if(!info_only && f_chunk->type == FLI_BRUN){
     if(!FliBrun(buff)){
       lFree(frame);
       return FALSE;
     }
   }
   else if(!info_only && f_chunk->type == FLI_COPY){
     for (j=0;j<ny;j++){
       p=Screen+(j)*pitch;
       memcpy(p,buff+j*nx,nx);
     }
   }
   buff += (f_chunk->size - 6);
 }
 XXIT:
 lFree(frame);
 frame2start=ftell(f);
 return TRUE;
}

BOOL FliColor(char *cbuf1){
 short i,j,pk,packets,change;
 unsigned char skip=0,colour;
 packets = *cbuf1++;
 packets += *cbuf1++ *256;
 if (packets<=0) return TRUE;
 for (pk=0; pk<packets; pk++ ){
   skip += *cbuf1++;
   change = *cbuf1++;
   if(change == 0)change=256;
   for(i=0; i<change; i++){
     colour = (unsigned char)*cbuf1++;
     p_red[skip] = (colour << 2);
     colour = (unsigned char)*cbuf1++;
     p_green[skip] = (colour << 2);
     colour = (unsigned char)*cbuf1++;
     p_blue[skip++] = (colour << 2);
   }
 }
 return TRUE;
}

static BOOL FliBrun(char *cbuf1){
 char size;
 unsigned char pk,packet;
 short i,line;
 char colour,*address;
 for (line=0;line<200;line++){
   address=(char *)(Screen+(line)*pitch);
   packet=*cbuf1++;
   for (pk=0; pk< packet; pk++){
     size=*cbuf1++;
     if(size > 0){
        colour = *cbuf1++;
        for(i = 0;i<size;i++)*address++ = colour;
     }
     else if(size < 0){
       for ( i = size; i<0; i++ )  *address++ = *cbuf1++;
     }
   }
 }
 return TRUE;
}

static BOOL FlcColor(char *buf, short type){
 short i,j,pk,packets,ci=0;
 unsigned char skip=0,change,colour;
 packets = *((short *)buf); buf += 2;
 for (pk=0; pk<packets; pk++ ) {
   skip = *((unsigned char *)buf); buf++;
   ci += skip;
   change = *((unsigned char *)buf); buf++;
   if (change == 0) change=255;
   for (i=0; i<=change; i++) {
     colour = *((unsigned char *)buf); buf++;
     if(type == FLC_COLOR)colour = colour << 2;
     p_red[ci] = colour;
     colour = *((unsigned char *)buf); buf++;
     if(type == FLC_COLOR)colour = colour << 2;
     p_green[ci] = colour;
     colour = (*(unsigned char *)buf); buf++;
     if(type == FLC_COLOR)colour = colour << 2;
     p_blue[ci] = colour;
     ci++;
   }
 }
 return TRUE;
}

static BOOL FlcBrun(char *buf){
 char size=0;
 unsigned char packet,*p,colour;
 short i,line,pk;
 for (line=0;line<ny;line++){
   p=Screen+(line)*pitch;
   packet = *((unsigned char *)buf); buf++;
   for(pk=0; pk<nx; pk+=abs(size)){
     size = *((char *)buf);  buf++;
     if(size > 0){
       colour = *((unsigned char *)buf); buf++;
       for (i=0; i<size; i++) *p++ = colour;
     }
     else{
       for (i = size; i<0; i++){
         colour = *((unsigned char *)buf); buf++;
         *p++ = colour;
       }
     }
   }
 }
 return TRUE;
}

static BOOL flc_unlccomp(char *buff){
 char size;
 unsigned char skip,eol;
 short pk,packet;
 short line,lines,word,words[256],size2;
 unsigned char *p;
 long j,i;
 lines = *((short *)buff); buff+=2;
 line=0;
 for(j = 0; j < lines; j++){
   LOOP:
   packet = *((short *)buff); buff += 2;
   if(packet < 0){
     if((packet & 0x4000) == 0x0400){
       eol = (unsigned char)(packet & 0x00ff);
     }
     line += abs(packet);
     goto LOOP;
   }
   p=Screen+(long)(line)*(long)pitch;
   for(pk=0; pk<packet; pk++){
     skip = *((unsigned char *)buff); buff++;
     p += (long)skip;
     size = *((char *)buff); buff++;
     size2=(short)size;
     if(size < 0){
       word = *((short *)buff); buff+=2;
       size = -size;
       for(i=0; i<size; i++){
         *((short *)p)=word; p += 2;
       }
     }
     else{
       size2=(short)size+(short)size;
       memcpy(p,buff,size2); buff += size2;  p += size2;
     }
   }
   line++;
 }
 return TRUE;
}

static BOOL fli_unlccomp(char *cbuf1){
 Fli_lc *flc;
 char *address;
 unsigned char skip,packet;
 short i,line;
 long s1,s2,size,pk;
 flc = (Fli_lc *)cbuf1;
 cbuf1 += 4;
 for( line = flc->skip; line < (short)(flc->skip)+(short)(flc->line); line++){
   packet = (unsigned char)(*cbuf1++);
   address=(char *)(Screen+((long)(line))*pitch);
   for(pk=0; pk<packet; pk++){
     skip = (unsigned) *cbuf1++;
     address += (unsigned)skip;
     size = (long)(*cbuf1++);
     if(size < 0){
       size = -size;
       memset(address,*cbuf1,size);
       address += size;
       cbuf1++;
     }
     else{
       memcpy(address,cbuf1,size);
       cbuf1 += size; address += size;
     }
   }
 }
 return TRUE;
}

#define C current_frame

static BOOL LoadAnimationIntoRam(void){
 Fli_frame i_frame;
 Flc_frame f_frame;
 ram_frame *temp;
 char *frame;
 long s;
 if(AnimationType == FLC_type){
   while(C < flc_cel.frames){
     if(RamFrames == NULL){
       RamFrames=(ram_frame *)lMalloc(sizeof(ram_frame));
       if(RamFrames == NULL)goto ERRX;
     }
     else{
       temp=(ram_frame *)lRealloc(RamFrames,sizeof(ram_frame)*(C+1));
       if(temp == NULL)goto ERRX;
       RamFrames=temp;
     }
     RamFrames[C].frame_header=NULL;
     RamFrames[C].frame=NULL;
     if(fread(&f_frame,sizeof(Flc_frame),1,FpF) != 1)goto ERRX;
     if(f_frame.type != FLCF_MAGIC)goto ERRX;
     s=f_frame.size-16;
     if((RamFrames[C].frame_header=(char *)
        lMalloc(sizeof(Flc_frame))) == NULL)goto ERRX;
     memcpy(RamFrames[C].frame_header,&f_frame,sizeof(Flc_frame));
     s=f_frame.size-16;
     if((frame = (char *)lMalloc(s)) == NULL)goto ERRX;
     RamFrames[C].frame=frame;
     if(fread(frame,1,s,FpF) != s)goto ERRX;
     C++;
   }
 }
 else{
   while(C < fli_cel.frame_count){
     if(RamFrames == NULL){
       RamFrames=(ram_frame *)lMalloc(sizeof(ram_frame));
       if(RamFrames == NULL)goto ERRX;
     }
     else{
       temp=(ram_frame *)lRealloc(RamFrames,sizeof(ram_frame)*(C+1));
       if(temp == NULL)goto ERRX;
       RamFrames=temp;
     }
     RamFrames[C].frame_header=NULL;
     RamFrames[C].frame=NULL;
     if(fread(&i_frame,sizeof(Fli_frame),1,FpF) != 1)goto ERRX;
     if(i_frame.type != FLIF_MAGIC)goto ERRX;
     s=i_frame.size-16;
          if((RamFrames[C].frame_header=(char *)
        lMalloc(sizeof(Fli_frame))) == NULL)goto ERRX;
     memcpy(RamFrames[C].frame_header,&i_frame,sizeof(Fli_frame));
     s=i_frame.size-16;
     if((frame = (char *)lMalloc(s)) == NULL)goto ERRX;
     RamFrames[C].frame=frame;
     if(fread(frame,1,s,FpF) != s)goto ERRX;
     C++;
   }
 }
 nframes=C;
 fclose(FpF); FpF=NULL;
 current_frame=0;
 return TRUE;
 ERRX:
 return FALSE;
}

#undef C

BOOL BlitNextFlcFrame(void){
 Flc_frame f_frame;
 Flc_chunk *f_chunk;
 char c,*frame,*buff;
 int i,j;
 long s;
 current_frame++;
 if(current_frame > flc_cel.frames){
   if(FpF != NULL){
     fseek(FpF,frame2start,SEEK_SET);   /* just in case use ours */
   }
   current_frame=1;
 }
 if(FpF != NULL){
   if(fread(&f_frame,sizeof(Flc_frame),1,FpF) != 1){
     return FALSE;
   }
   if(f_frame.type != FLCF_MAGIC){
     return FALSE;
   }
   s=f_frame.size-16;
   if((frame = (char *)lMalloc(s)) == NULL)return FALSE;
   if(fread(frame,1,s,FpF) != s)return FALSE;
 }
 else if(RamFrames != NULL){
   memcpy(&f_frame,RamFrames[current_frame-1].frame_header,sizeof(Flc_frame));
   frame=RamFrames[current_frame-1].frame;
 }
 else{
   return FALSE;
 }
 buff=frame;
 for(i=0;i<f_frame.chunks;i++){
   f_chunk=(Flc_chunk *)buff;  buff += 6;
   if(f_chunk->type == FLC_SS2){   /* found cell */
     if(flc_unlccomp(buff) < 0){
       lFree(frame);
       return FALSE;
     }
   }
   else if(f_chunk->type == FLC_LC){
     if(fli_unlccomp(buff) < 0){
       lFree(frame);
       return FALSE;
     }
   }
   else if(f_chunk->type == FLC_COPY){
     unsigned char *p;
     for (j=0;j<ny;j++){
       p=Screen+(j)*pitch;
       memcpy(p,buff+j*nx,nx);
     }
   }
   buff += (f_chunk->size - 6);
 }
 if(FpF != NULL)lFree(frame);
 return TRUE;
}

BOOL BlitNextFliFrame(void){
 Fli_frame f_frame;
 Fli_chunk *f_chunk;
 char c,*frame,*buff;
 int i,j;
 long s;
 current_frame++;
 if(current_frame > fli_cel.frame_count){
   if(FpF != NULL){
     fseek(FpF,frame2start,SEEK_SET);   /* but just in case user ours */
   }
   current_frame=1;
 }
 if(FpF != NULL){
   if(fread(&f_frame,sizeof(Fli_frame),1,FpF) != 1){
     return FALSE;
   }
   if(f_frame.type != FLIF_MAGIC){
     return FALSE;
   }
   s=f_frame.size-16;
   if((frame = (char *)lMalloc(s)) == NULL)return FALSE;
   if(fread(frame,1,s,FpF) != s)return FALSE;
 }
 else if(RamFrames != NULL){
   memcpy(&f_frame,RamFrames[current_frame-1].frame_header,sizeof(Fli_frame));
   frame=RamFrames[current_frame-1].frame;
 }
 else{
   return FALSE;
 }
 buff=frame;
 for(i=0;i<f_frame.chunks;i++){
   f_chunk=(Fli_chunk *)buff;  buff += 6;
   if(f_chunk->type == FLI_LC){
     if(fli_unlccomp(buff) < 0){
       lFree(frame);
       return FALSE;
     }
   }
   else if(f_chunk->type == FLI_COPY){
     unsigned char *p;
     for (j=0;j<ny;j++){
       p=Screen+(j)*pitch;
       memcpy(p,buff+j*nx,nx);
     }
   }
   buff += (f_chunk->size - 6);
 }
 if(FpF != NULL)lFree(frame);
 return TRUE;
}

BOOL BlitFirstFlxFrame(char *CurrentFile){
 int i,j,xc,yc,xs,ys,nf;
 char temp[128];
 long imagesize;
 FreeAnimation();
 strcpy(temp,CurrentFile);
 if(AnimationType == FLC_type){
   if(!OpenFLC(temp,1)){
     if(FpF != NULL)fclose(FpF); FpF=NULL;
     return FALSE;
   }
 }
 else{
   if(!OpenFLI(temp,1)){
     if(FpF != NULL)fclose(FpF); FpF=NULL;
     return FALSE;
   }
 }
 if(FpF != NULL)fclose(FpF); FpF=NULL;

 imagesize = (DWORD)nx*(DWORD)ny;
 if(AnimationType == FLC_type){
   if(!OpenFLC(temp,0)){
     FreeAnimation();
     return FALSE;
   }
 }
 else{
   if(!OpenFLI(temp,0)){
     FreeAnimation();
     return FALSE;
   }
 }
 current_frame=0;

 if(AnimationType == FLC_type)nf=flc_cel.frames;
 else                         nf=fli_cel.frame_count;
 max_frames=nf;
 AnimationOpen=TRUE;
 if(bInRam){
   LoadAnimationIntoRam();
   MessageBeep(MB_OK);
 }
 return TRUE;
}
