#include <windows.h>
#include <commdlg.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <math.h>

#include "resource.h"

#define lMalloc(s)    LocalAlloc(LMEM_FIXED,s)
#define lFree(p)      LocalFree((HLOCAL)p)
#define lRealloc(p,s) LocalReAlloc((HLOCAL)p,s,LMEM_MOVEABLE)

//
//  Functions in other files whichload the next image in the movie or flic.
//

extern BOOL BlitFirstFlxFrameL(char *);
extern BOOL BlitNextFlcFrameL(void);
extern BOOL NextFliFrameL(void);
extern BOOL BlitFirstFlxFrameR(char *);
extern BOOL BlitNextFlcFrameR(void);
extern BOOL NextFliFrameR(void);
extern BOOL BlitFirstAVIFrame(char *);
extern BOOL BlitNextAviFrame(void);

#define FLC_type 0          //  Identify the type of animation file
#define FLI_type 1          //  (found from its filename extension
#define AVI_type 2

//
// Variables used globally
//

unsigned char   *ScreenL=NULL;
unsigned char   *ScreenR=NULL;
unsigned char   *Screen4=NULL;
long            pitch,x_size,y_size;

long            AnimationType=FLC_type;     // Default to FLC type animations

BOOL            bRestart=FALSE;
BOOL            bInRam=FALSE;               // Read one frame from disk and
                                            // show it be fore reading the
                                            // next, or read all the frames
                                            // into memory in their compressed
                                            // form and decompress to SCREEN
                                            // this shoudl be a little faster
                                            // but Windows has excellent
                                            // file caching that makes this
                                            // virtually unnecesary.

unsigned char   p_redL[255],                // The 8 bit color palette from
                p_greenL[255],              // the flic or movie file. In this
                p_blueL[255];               // example of the use of DirectDraw
                                            // we will exclusively use the
                                            // 8 bit paletted display mode.
unsigned char   p_redR[255],                // The 8 bit color palette from
                p_greenR[255],              // the flic or movie file. In this
                p_blueR[255];               // example of the use of DirectDraw
                                            // we will exclusively use the
                                            // 8 bit paletted display mode.

//
// Variables used locally - All the DirectDraw specific code is in this file
//

static HWND                hWnd;                   // The applications window

static long                lTimer          = 0;
static BOOL                bActive         = FALSE;
static BOOL                bIsInitialized  = FALSE;
static BOOL                bUseTimer       = FALSE;
static BOOL                bStereo         = FALSE;
static BOOL                bStereoDraw     = FALSE;
static PALETTEENTRY        pe[256];
static HPALETTE        ghpalOld, ghPalette = NULL;

//
// Prototypes for local functions
//

void DrawScene(HWND);
static GLvoid initialize(HWND);
static BOOL bSetupPixelFormat(HDC);
static void UpdateFrame(void);
static BOOL Initialize(HINSTANCE, int);
static BOOL ReadFlicIntoSurfaces(BOOL, char *, char *);
static int  SelectFileDialog(char *, char *, HWND, HINSTANCE );
static int  SelectFile(char *, char *, HWND, HINSTANCE );
static LRESULT CALLBACK WindowProc(HWND,UINT, WPARAM, LPARAM);

//////////////// The application's main function ////////////////////////////

int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,
                     LPSTR lpCmdLine,int nCmdShow)
{
    MSG  msg;
    char FlicName[256],RightFlicName[256];
    //
    // Get the filename of the file for the flic or movie we wish to
    // play, use the filename extension to determine the file type.
    //
    RESTART:
    if(bRestart){
      if(ScreenL != NULL)lFree(ScreenL);
      if(ScreenR != NULL)lFree(ScreenR);
      if(Screen4 != NULL)lFree(Screen4);
      ScreenL=ScreenR=Screen4=NULL;
      bRestart=FALSE;
    }
    if(!SelectFileDialog(FlicName,
                       "Flic files (*.FLC *.FLI *.AVI)|*.fl?;*.avi|",
                       NULL,hInstance))return 0;
    if     (strstr(strupr(FlicName),".FLI") != NULL)AnimationType=FLI_type;
    else if(strstr(strupr(FlicName),".AVI") != NULL)AnimationType=AVI_type;
    else                                            AnimationType=FLC_type;
    //
    if     (AnimationType == FLC_type){x_size=640; y_size=480;}
    else if(AnimationType == AVI_type){x_size=640; y_size=480;}
    else                              {x_size=320; y_size=200;}
    if(bStereo)if(!SelectFile(RightFlicName,
                              "Flic files (*.FLC *.FLI *.AVI)|*.fl?;*.avi|",
                               NULL,hInstance))return 0;

    //
    // Initialize the application by creating the DirectDraw objects and
    // surfaces and then switching the display adapter into the correct mode.
    // Either 640 x 480  or  320 by 200 (both resloutions use 256 colors).
    //
    if(!Initialize(hInstance,nCmdShow))return FALSE;
    //
    // Read the first frame from the flic or movie into display memory
    //
    ReadFlicIntoSurfaces(TRUE,FlicName,RightFlicName);
    //
    // Update the display surface (After this point the first frame should
    // be visible irrespective of whether double buffering is in use or
    // the image is being written directly into video memory.
    //
    UpdateFrame();
    //
    // If a timer is being used to display the frames at regular times
    // create it now.
    //
    if(bUseTimer){
      lTimer=SetTimer(hWnd,1,33,NULL);
    }
    //
    // Main message loop
    //
    while(1){
        if(PeekMessage(&msg,NULL,0,0,PM_NOREMOVE)){
            if(!GetMessage(&msg,NULL,0,0)) {
                if(bRestart)goto RESTART;
                return msg.wParam;
            }
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        //
        // If not using a timer - draw the next frame.
        //
        else if(bActive && !bUseTimer){
          ReadFlicIntoSurfaces(FALSE,NULL,NULL);
          UpdateFrame();
        }
        else{
          WaitMessage();
        }
    }
    if(bRestart)goto RESTART;
    return 0;
}

/////////////////////////// Initialize the application //////////////////////

static BOOL Initialize(HINSTANCE hInstance,int nCmdShow){
 WNDCLASS            wc;
 //
 // Register a class for a Windows window
 //
 wc.style=          CS_HREDRAW|CS_VREDRAW;
 wc.lpfnWndProc=    WindowProc;
 wc.cbClsExtra=     0;
 wc.cbWndExtra=     0;
 wc.hInstance=      hInstance;
 wc.hIcon=          LoadIcon(hInstance,MAKEINTRESOURCE(IDI_ICON1));
 wc.hCursor=        LoadCursor(NULL,IDC_ARROW);
 wc.hbrBackground=  NULL;
 wc.lpszMenuName=   NULL;
 wc.lpszClassName=  "DDPLAY";
 RegisterClass(&wc);
 //
 // Make a window that fills the desktop - this window will not be
 // seen because we will use DirectDraw to draw the movie so that it
 // fills the screen. The primary function of this window is to
 // process messages.
 //
 hWnd=CreateWindowEx(
     0,
     "DDPLAY",
     "DDPLAY",
     WS_POPUP,
     0,
     0,
     GetSystemMetrics(SM_CXSCREEN),
     GetSystemMetrics(SM_CYSCREEN),
     NULL,
     NULL,
     hInstance,
     NULL );
 if(!hWnd)return FALSE;
 ShowWindow(hWnd,nCmdShow);
 UpdateWindow(hWnd);

 bIsInitialized = TRUE;
 //
 // All the necessary DirectDraw objects have been created and initialized.
 //
 return TRUE;
}

///////////////////// The window function procedure /////////////////////////

static LRESULT CALLBACK WindowProc(HWND hWnd,UINT message,
                                   WPARAM wParam,LPARAM lParam){
    int speed;
    switch(message){
  case WM_CREATE:{
            HGLRC hRC;
            HDC hDC;
            hDC = GetDC(hWnd);
            bSetupPixelFormat(hDC);
            hRC = wglCreateContext( hDC );
            wglMakeCurrent( hDC, hRC );
            initialize(hWnd);
          }
          break;
        case WM_DESTROY:{
            HGLRC  hRC;
            HDC    hDC;
            if(lTimer > 0)KillTimer(hWnd,lTimer);
            hRC = wglGetCurrentContext();
            hDC = wglGetCurrentDC();
            wglMakeCurrent(NULL, NULL);
            if (hRC != NULL)wglDeleteContext(hRC);
            if (hDC != NULL)ReleaseDC(hWnd, hDC);
          }
          PostQuitMessage(0);
          break;
        case WM_PAINT:{
            HDC hDC;
            PAINTSTRUCT  ps;
            hDC = BeginPaint(hWnd, &ps);
            DrawScene(hWnd);
            EndPaint(hWnd, &ps); 
          }
          break;
        case WM_TIMER:
          if(bActive){
            //
            // The timer has gone off - read the next image (frame)
            // into the DirectDraw surface and display it.
            //
            ReadFlicIntoSurfaces(FALSE,NULL,NULL);
            UpdateFrame();
          }
          break;
        case WM_ACTIVATE:
            bActive = wParam;
            break;
        case WM_SETCURSOR:
            SetCursor(NULL);
            break;
        case WM_KEYDOWN:
            switch(wParam) {
                //
                // Use the ESCAPE or F12 key to terminate the application.
                //
                case VK_INSERT:
                case VK_ESCAPE:
                case VK_F12:
                  bRestart=TRUE;
                  PostMessage(hWnd,WM_CLOSE,0,0);
                  break;
                case VK_F1: speed=1;    goto SPEED;
                case VK_F2: speed=5;    goto SPEED;
                case VK_F3: speed=10;   goto SPEED;
                case VK_F4: speed=20;   goto SPEED;
                case VK_F5: speed=33;   goto SPEED;
                case VK_F6: speed=66;   goto SPEED;
                case VK_F7: speed=132;  goto SPEED;
                case VK_F8: speed=266;  goto SPEED;
                  //
                  // If a timer is in use use these keys to
                  // change the display rate.
                  //
                  SPEED:
                  if(lTimer > 0){
                    KillTimer(hWnd,lTimer);
                    lTimer=SetTimer(hWnd,1,speed,NULL);
                  }
                  break;
            }
            break;
    }
    return DefWindowProc(hWnd,message,wParam,lParam);
}

//////////////////////// The remaining DirectDraw functions /////////////////

static BOOL ReadFlicIntoSurfaces(BOOL first, char *FlicName, char *RightFlicName){
 int i;
 //
 // If this is the first time this function is called the first frame
 // in the animation is displayed.
 //

 if(first){
   if(ScreenL != NULL)lFree(ScreenL);
   ScreenL=(unsigned char *)lMalloc(x_size*y_size);
   if(ScreenL == NULL)return FALSE;
   if(ScreenR != NULL)lFree(ScreenR);
   ScreenR=(unsigned char *)lMalloc(x_size*y_size);
   if(ScreenR == NULL)return FALSE;
   pitch=x_size;
   if(Screen4 != NULL)lFree(Screen4);
   Screen4=(unsigned char *)lMalloc(x_size*3*y_size);
   if(Screen4 == NULL)return FALSE;
   //
   // Copy the first frame into video memory.
   //
   if(AnimationType == AVI_type)BlitFirstAviFrame(FlicName);
   else {
     BlitFirstFlxFrameL(FlicName);
     if(bStereo)BlitFirstFlxFrameR(RightFlicName);
   }
 }
 else{
   //
   // Read the next image into video memory
   //
   if     (AnimationType == AVI_type)BlitNextAviFrame();
   else if(AnimationType == FLC_type){
     BlitNextFlcFrameL();
     if(bStereo)BlitNextFlcFrameR();
   }
   else{
     BlitNextFliFrameL();
     if(bStereo)BlitNextFliFrameR();
   }
   //
   // Unlock the memory buffer after we have written to it.
   //
 }
 return TRUE;
}

static void UpdateFrame(void){
 InvalidateRect(hWnd, NULL, FALSE);
}


/////////////////////////////////////////////////////////////////////////////

//
// The last two functions provide the "User Interface", it is based on
// an expanded "Explorer Style File Open Dialog" that provides extra check
// boxes that enable the user to set the flags:
//                                             bUserTimer,
//                                             bInRam

static unsigned CALLBACK NameHookProc(HWND hwnd,UINT msg,
                              WPARAM wparam,LPARAM lparam){
 //
 // Callback function used by the GetOpenFileName API
 //
 switch( msg ) {
   case WM_INITDIALOG:
     if(bInRam)SendDlgItemMessage(hwnd,IDC_CHECK1,BM_SETCHECK,1,0);
     if(bStereo)SendDlgItemMessage(hwnd,IDC_CHECK2,BM_SETCHECK,1,0);
     if(bUseTimer)SendDlgItemMessage(hwnd,IDC_CHECK3,BM_SETCHECK,1,0);
     return FALSE;
   case WM_NOTIFY:{
       LPOFNOTIFY lp=(LPOFNOTIFY)lparam;
       if(lp->hdr.code == CDN_FILEOK){
         //
         // OK is pressed get the flags
         //
         if(SendDlgItemMessage(hwnd,IDC_CHECK1,BM_GETCHECK,0,0))
              bInRam=TRUE;
         else bInRam=FALSE;
         if(SendDlgItemMessage(hwnd,IDC_CHECK2,BM_GETCHECK,0,0))
              bStereo=TRUE;
         else bStereo=FALSE;
         if(SendDlgItemMessage(hwnd,IDC_CHECK3,BM_GETCHECK,0,0))
              bUseTimer=TRUE;
         else bUseTimer=FALSE;
       }
     }
     break;
   default: break;
 }
 return FALSE;
}

static BOOL SelectFileDialog(char *szfile, char *szfilter,
                           HWND parent, HINSTANCE hInst){
 //
 // Get a filename or the movie or flic file by setting up an OPENFILENAME
 // structure and calling the API "GetOpenFileName"
 //
 int i;
 OPENFILENAME ofn;
 char szFilter[80];
 strcpy(szFilter,szfilter);
 i=0; while(szFilter[i] != '\0'){
   if(szFilter[i] == '|')szFilter[i]='\0'; i++;
 }
 memset(&ofn,0,sizeof(OPENFILENAME));
 ofn.lStructSize=sizeof(OPENFILENAME);
 ofn.hwndOwner=parent;
 ofn.lpstrFilter=szFilter;
 ofn.nFilterIndex=0;
 szfile[0]='\0';
 ofn.lpstrFile=szfile;
 ofn.nMaxFile=255;
 ofn.lpstrFileTitle=NULL;
 ofn.nMaxFileTitle=0;
 ofn.lpstrInitialDir=NULL;
 ofn.lpstrTitle="Open Movie (For Stereo Choose Left Eye File)";
 ofn.hInstance=hInst;
 ofn.lpfnHook=NameHookProc;
 ofn.lpTemplateName=MAKEINTRESOURCE(IDD_DIALOG1);
 ofn.Flags=OFN_HIDEREADONLY|OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST|
           OFN_EXPLORER|
           OFN_ENABLEHOOK|OFN_ENABLETEMPLATE|OFN_NONETWORKBUTTON;
 return GetOpenFileName(&ofn);
}

static BOOL SelectFile(char *szfile, char *szfilter,
                        HWND parent, HINSTANCE hInst){
 int i;
 OPENFILENAME ofn;
 char szFilter[80];
 strcpy(szFilter,szfilter);
 i=0; while(szFilter[i] != '\0'){
   if(szFilter[i] == '|')szFilter[i]='\0'; i++;
 }
 memset(&ofn,0,sizeof(OPENFILENAME));
 ofn.lStructSize=sizeof(OPENFILENAME);
 ofn.hwndOwner=parent;
 ofn.lpstrFilter=szFilter;
 ofn.nFilterIndex=0;
 szfile[0]='\0';
 ofn.lpstrFile=szfile;
 ofn.nMaxFile=255;
 ofn.lpstrFileTitle=NULL;
 ofn.nMaxFileTitle=0;
 ofn.lpstrInitialDir=NULL;
 ofn.lpstrTitle="Select Flic for Right Eye";
 ofn.hInstance=hInst;
 ofn.Flags=OFN_HIDEREADONLY|OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST|
           OFN_EXPLORER|
           OFN_NONETWORKBUTTON;
 return GetOpenFileName(&ofn);
}

static BOOL bSetupPixelFormat(HDC hDC){
  static PIXELFORMATDESCRIPTOR pfd = {
    sizeof(PIXELFORMATDESCRIPTOR),    // size of this pfd
      1,                              // version number
      PFD_DRAW_TO_WINDOW |            // support window
      PFD_SUPPORT_OPENGL |            // support OpenGL
      PFD_DOUBLEBUFFER,               // double buffered
      PFD_TYPE_RGBA,                  // RGBA type
      24,                             // 24-bit color depth
      0, 0, 0, 0, 0, 0,               // color bits ignored
      0,                              // no alpha buffer
      0,                              // shift bit ignored
      0,                              // no accumulation buffer
      0, 0, 0, 0,                     // accum bits ignored
      32,                             // 32-bit z-buffer
      0,                              // no stencil buffer
      0,                              // no auxiliary buffer
      PFD_MAIN_PLANE,                 // main layer
      0,                              // reserved
      0, 0, 0                         // layer masks ignored
      };
  int pixelformat;
  if (bStereo){
    pfd.dwFlags |= PFD_STEREO;
  }
  if ( (pixelformat = ChoosePixelFormat(hDC, &pfd)) == 0 ) {
    MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK);
    return FALSE;
  }
  if (SetPixelFormat(hDC, pixelformat, &pfd) == FALSE) {
    MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK);
    return FALSE;
  }
  if(bStereo){
    pixelformat = GetPixelFormat (hDC);
    DescribePixelFormat (hDC, pixelformat, sizeof (PIXELFORMATDESCRIPTOR), &pfd);
    if (!(pfd.dwFlags & PFD_STEREO)){
      MessageBox (NULL, "OFX Movie Player: Stereo mode not supported by this hardware.", "Stereo Error", MB_OK | MB_ICONINFORMATION);
      bStereoDraw=FALSE;
    }
    else bStereoDraw=TRUE;
  }
  return TRUE;
}

static GLvoid initialize(HWND hWnd){
 glClearColor(0.0,0.0,0.0,1.0);
 glClearDepth(1.0);
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity();
 glOrtho(0.0,1.0,0.0,1.0,-1.0,1.0);
 glMatrixMode( GL_MODELVIEW );
 //glRasterPos2f(0.0,0.0);
 glRasterPos2i(0,0);
 glPixelZoom((GLfloat)GetSystemMetrics(SM_CXSCREEN)/(GLfloat)x_size,
             (GLfloat)GetSystemMetrics(SM_CYSCREEN)/(GLfloat)y_size);
 return;
}

void DrawScene(HWND hWnd){
 HDC hDC;
 if(Screen4 != NULL){
   int i,j,k;
   unsigned char id;
   k=0;
   //glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   if(bStereoDraw)glDrawBuffer (GL_BACK_LEFT);
   for(i=y_size-1;i>=0;i--)
   for(j=0;j<x_size;j++){
     id=ScreenL[i*x_size+j];
     *(Screen4+k) = p_redL[id];   k++;
     *(Screen4+k) = p_greenL[id]; k++;
     *(Screen4+k) = p_blueL[id];  k++;
   }
   glDrawPixels((GLsizei)x_size,(GLsizei)y_size,GL_RGB,GL_UNSIGNED_BYTE,(GLvoid *)Screen4);
   if(bStereoDraw){
     glClear (GL_DEPTH_BUFFER_BIT);
     glFlush ();
     glDrawBuffer (GL_BACK_RIGHT);
     k=0;
     for(i=y_size-1;i>=0;i--)
     for(j=0;j<x_size;j++){
       id=ScreenR[i*x_size+j];
       *(Screen4+k) = p_redR[id];   k++;
       *(Screen4+k) = p_greenR[id]; k++;
       *(Screen4+k) = p_blueR[id];  k++;
     }
     glDrawPixels((GLsizei)x_size,(GLsizei)y_size,GL_RGB,GL_UNSIGNED_BYTE,(GLvoid *)Screen4);
   }
 }
 glFlush();
 glFinish();
 hDC = wglGetCurrentDC();
 SwapBuffers(hDC);
 return;
}
