/*=========================================================================

  Program:   Visualization Toolkit
  Module:    vtkLWObject.hh
  Language:  C++
  Date:      08/31/95
  Version:   1.11


Copyright (c) 1993-1995 Ken Martin, Will Schroeder, Bill Lorensen.

This software is copyrighted by Ken Martin, Will Schroeder and Bill Lorensen.
The following terms apply to all files associated with the software unless
explicitly disclaimed in individual files. This copyright specifically does
not apply to the related textbook "The Visualization Toolkit" ISBN
013199837-4 published by Prentice Hall which is covered by its own copyright.

The authors hereby grant permission to use, copy, and distribute this
software and its documentation for any purpose, provided that existing
copyright notices are retained in all copies and that this notice is included
verbatim in any distributions. Additionally, the authors grant permission to
modify this software and its documentation for any purpose, provided that
such modifications are not distributed without the explicit consent of the
authors and that existing copyright notices are retained in all copies. Some
of the algorithms implemented by this software are patented, observe all
applicable patent law.

IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY DERIVATIVES THEREOF,
EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE IS PROVIDED ON AN
"AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.


=========================================================================*/

// .NAME vtkLWObject - abstract base class for visualization library
// .SECTION Description
// vtkLWObject is the base class for many objects that use multiple 
// inheritance.  vtkLWObject is a "light weight" version of vtkObject, 
// whose sole function is to resolve issues of multiple inheritance.
// (NOTE: This was done because the alternative, virtual base classes,
// causes many problems with current C++ compilers. Also, the ability to
// "cast down" is not possible with virtual base classes.)
//
// This class reimplements some of the methods and instance variables
// found in vtkObject. It uses the same naming scheme, but prepends each with
// a "_" character.

#ifndef __vtkLWObject_hh
#define __vtkLWObject_hh

#include <iostream.h>
#include "vtkTimeStamp.hh"
#include "vtkIndent.hh"

// Replacements for the standard SetGet macros
#define vtk_DebugMacro(x) \
  if (_Debug) cerr << "Debug: In " __FILE__ << ", line " << __LINE__ << "\n" << this->_GetClassName() << " (" << this << "): " x << "\n\n"

#define vtk_WarningMacro(x) \
  cerr << "Warning: In " __FILE__ << ", line " << __LINE__ << "\n" << this->_GetClassName() << " (" << this << "): " x << "\n\n"

#define vtk_ErrorMacro(x) \
  cerr << "ERROR In " __FILE__ << ", line " << __LINE__ << "\n" << this->_GetClassName() << " (" << this << "): " x << "\n\n"


class vtkLWObject 
{
public:
  vtkLWObject();
  virtual ~vtkLWObject() {};
  virtual char *_GetClassName() {return "vtkLWObject";};

  void _Modified();
  virtual unsigned long int _GetMTime();

  void _PrintSelf(ostream&, vtkIndent) {};

  void _DebugOn();
  void _DebugOff();

protected:
  vtkTimeStamp _MTime; // Keep track of modification time
  int _Debug;

};

// Description:
// Construct object.
inline vtkLWObject::vtkLWObject()
{
  this->_MTime.Modified();
  this->_Debug = 0;
}

// Description:
// Update the modification time for this object.
inline void vtkLWObject::_Modified()
{
  this->_MTime.Modified();
}

// Description:
// Return the modification for this object.
inline unsigned long int vtkLWObject::_GetMTime() 
{
  return this->_MTime.GetMTime();
}

// Description:
// Turn debug printout on.
inline void vtkLWObject::_DebugOn()
{
  this->_Debug = 1;
}

// Description:
// Turn debug printout off.
inline void vtkLWObject::_DebugOff()
{
  this->_Debug = 0;
}

#endif

